<?php
/**
 * Classroom.class.php
 * 
 * This file contains the gateway to openmeetings
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * @author Heiko Dostert <dosterth@uni-trier.de>
 * @package portaLE
 * @copyright Universitt Trier, Koordinationsstelle E-Learning, 2010
 * @version 1.0.0
 * @version 1.0.2 - 22.11.2010 - Jrg Rpke <roepke@uni-trier.de>
 * @version 1.1.0 - 01.03.2011 - Paul C. Sommerhoff <somm4702@uni-trier.de>
 */


require_once dirname(__FILE__) . '/../config/config_portale.inc.php';

class OmGateway {
	var $server_name;
	var $server;
	var $session_id;
	var $room_id;

	public function __construct($server, $room_id = false){
		global $OM_SERVERS;
		if(empty($server))
			throw new PortaleException(_("Fehler beim Zugriff auf die Openmeetings-Server: Fehlender Parameter"));
		if(!isset($OM_SERVERS))
			throw new PortaleException(_("Fehler beim Zugriff auf Openmeetings: Keinen Server gefunden"));
		if(empty($OM_SERVERS))
			throw new PortaleException(_("Fehler beim Zugriff auf Openmeetings: Keinen Server gefunden"));
		if(!isset($OM_SERVERS[$server]))
			throw new PortaleException(_("Fehler beim Zugriff auf die Openmeetings: Server kann nicht gefunden werden"));
		$this->server_name = $server;
		$this->server = $OM_SERVERS[$server];
		try{
			$client_userService = new SoapClient("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/UserService?wsdl");
			$result = $client_userService->getSession();
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Der Konstruktor der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Der Openmeetings-Server ist nicht verfgbar. Bitte wenden Sie sich den Systemadministrator."));
		}
		$this->session_id = $result->return->session_id;
		$params = array(
			'SID' => $this->session_id,
			'username' => $this->server['ADMIN'],
			'userpass' => $this->server['ADMIN_PW']
		);
		try{
			$result = $client_userService->loginUser($params);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Der Konstruktor der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich aun dden Systemadministrator."));
		}
		$returnValue = $result->return;
		if($returnValue<=0){  //error
			$params = array(
					'SID' => $this->session_id,
					'errorid' => $returnValue,
					'language_id' => '3'  //english:1; german:2; german (studip):3
			);
			$error = $client_userService->getErrorByCode($params);
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Der Konstruktor der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$error->return->errmessage);
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich an den Sytemadministrator."));
		}
		if($room_id !== false)
    		$this->room_id = $room_id; 
	}  //End __construct
  
  
	public function createRoomWithMod($name, $roomtypes_id, $numberOfPartizipants, $comment, $date, $length){
		$params = array(
			'SID' => $this->session_id,
			'name' => utf8_encode($name),
			'roomtypes_id' => $roomtypes_id,
			'comment' => utf8_encode($comment),
			'numberOfPartizipants' => $numberOfPartizipants,
			'ispublic' => true,
			'appointment' => false, 
			'isDemoRoom' => false, 
			'demoTime' => 0, 
			'isModeratedRoom' => true
		);
		try{
			$client_roomService = new SoapClient("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/RoomService?wsdl", array('encoding'=>'utf-8'));
			$result = $client_roomService->addRoomWithModeration($params);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode createRoomWithMod() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich an den Systemadministrator."));
		}
		if($result->return == -1)  //error
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server."));
		$this->room_id = $result->return;

		//enter data into db:
		$options = array("location" => $this->server['SHARINGINTERFACE_LOCATION'],       
						"uri" => $this->server['SHARINGINTERFACE_URI'],                  
						"style" => $this->server['SHARINGINTERFACE_STYLE'],                                
						"use" => $this->server['SHARINGINTERFACE_USE']);
		$mysql_startdate = date( 'Y-m-d H:i:s', $date);
		$mysql_enddate = date( 'Y-m-d H:i:s', ($date + $length + TIME_OFFSET));
		$mysql_archivedate = date( 'Y-m-d H:i:s', ($date + $length + STORAGE_TIME));
		try{
			$om_interface = new SoapClient(null, $options);
			$om_interface->setRoom($this->room_id, $mysql_startdate, $mysql_enddate, $mysql_archivedate, $this->server['INSTALLATION'], $this->server['SEC_KEY']);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode createRoomWithMod() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf die Datenbank: ").$fault->__toString());
		}
			
		return $this->room_id;
	}

	
	public function createUserAndGenerateRoomHash($givenname, $surname, $isMod, $username=''){
		//test, if user has already entered the room or if max number of participants is reached:
		//geht noch nicht, da offenbar Bug in SOAP-Funktion getRoomWithCurrentUsersById: currentuser-Feld immer null (laut OM-Mailinglist wird daran gearbeitet)
/*		$client_roomService = new nusoap_client_om("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/RoomService?wsdl", true);
		$err = $client_roomService->getError();
		if ($err)
			throw new PortaleException(_("Fehler beim Zugriff auf die Openmeetings-Server:").$err);
		$params = array(
			'SID' => $this->session_id,
			'rooms_id' => $this->room_id,
		); 
		$result = $client_roomService->call('getRoomWithCurrentUsersById',$params);
		if ($client_roomService->fault)
			throw new PortaleException(_("Fehler beim Zugriff auf die Openmeetings-Server"));
		else {
			$err = $client_roomService->getError();
			if ($err)
				throw new PortaleException(_("Fehler beim Zugriff auf die Openmeetings-Server"));
			else{
				$room = $result["return"];
				echo "Raum-ID: ".$this->room_id;
				var_dump($room);
				echo "aktuelle User:<br/>";
				var_dump($room['currentusers']);	
			}
		}*/
		
	    //check if there is place in the room (just persons with tutor permission can enter full rooms):
	    if(!$isMod){
	    	$q = "SELECT numberOfParticipants FROM portaLE_conferences WHERE server = '$this->server_name' AND rooms_id = '$this->room_id'";
			$data = DBManager::Get()->query($q)->fetch();
			if($data === false)
				return false;
			$maxSize = $data['numberOfParticipants'];
			$options = array("location" => $this->server['SHARINGINTERFACE_LOCATION'],       
						"uri" => $this->server['SHARINGINTERFACE_URI'],                  
						"style" => $this->server['SHARINGINTERFACE_STYLE'],                                
						"use" => $this->server['SHARINGINTERFACE_USE']);
			try{
				$om_interface = new SoapClient(null, $options);
				$size = $om_interface->getRoomSize($this->room_id, $this->server['SEC_KEY']);
				if($size === false){
					if(defined('MAIL_ADMIN'))
						mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode getRoomSize() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: Falsche Passwort bei SOAP-Aufruf'));
					throw new PortaleException(_("Fehler beim Zugriff auf die Datenbank. Bitte wenden Sie sich an den Systemadministrator."));
				}
				if($size >= $maxSize)
					throw new PortaleException(_("Sie knnen den virtuellen Raum leider nicht betreten, da die maximale Teilnehmerzahl bereits erreicht ist."));		
			}
			catch(SoapFault $fault){
				if(defined('MAIL_ADMIN'))
					mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode getRoomSize() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
				throw new PortaleException(_("Fehler beim Zugriff auf die Datenbank. Bitte wenden Sie sich an den Systemadministrator."));
			}
	    }
		
		// Pfad fr Userbild erzeugen 
		if((file_exists($GLOBALS[$ABSOLUTE_PATH_STUDIP]."pictures/user/".get_userid($username)."_medium.png")))
			$my_picture_path = $GLOBALS[ABSOLUTE_URI_STUDIP]."pictures/user/".get_userid($username)."_medium.png";
		else
			$my_picture_path = $GLOBALS[ABSOLUTE_URI_STUDIP]."pictures/user/nobody_medium.png";
		
		$params = array(
			'SID' => $this->session_id,
			'username' => utf8_encode($username),
			'firstname' => utf8_encode($givenname),
			'lastname' => utf8_encode($surname),
			'profilePictureUrl' => $my_picture_path,
			'email' => '',
			'externalUserId' => '1234',
			//'externalUserType' => 'studip',
			'externalUserType' => $this->server['INSTALLATION'],
			'room_id' => $this->room_id,
			'becomeModeratorAsInt' => $isMod,
			'showAudioVideoTestAsInt' => $isMod
		);
		try{
			$client_userService = new SoapClient("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/UserService?wsdl");
			$result = $client_userService->setUserObjectAndGenerateRoomHash($params);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode createUserAndGenerateRoomHash() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich an den Systemadministrator."));
		}
		return "http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/main.swf8.swf?secureHash=". $result->return."&lzproxied=solo&language=3";
	}  //Ende createUserAndGenerateRoomHash()
  
	
	public function deleteRoom($room_id){
		$params = array(
			'SID' => $this->session_id,
			'rooms_id' => $room_id
		);
		try{
			$client_roomService = new SoapClient("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/RoomService?wsdl");
			$result = $client_roomService->deleteRoom($params);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode deleteRoom() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich an den Systemadministrator."));
		}
        
        //delete data record from db:
		$options = array("location" => $this->server['SHARINGINTERFACE_LOCATION'],       
						"uri" => $this->server['SHARINGINTERFACE_URI'],                  
						"style" => $this->server['SHARINGINTERFACE_STYLE'],                                
						"use" => $this->server['SHARINGINTERFACE_USE']);
		try{
			$om_interface = new SoapClient(null, $options);
			$om_interface->deleteRoom($room_id, $this->server['SEC_KEY']);
		}
		catch(SoapFault $fault){
			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode deleteRoom() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
			throw new PortaleException(_("Fehler beim Zugriff auf die Datenbank. Bitte wenden Sie sich an den Systemadministrator."));
		}
	}

  
  	public function kickAll(){
  		$params = array(
			'SID_Admin' => $this->session_id,
			'room_id' => $this->room_id);
  		try{
			$client_roomService = new SoapClient("http://".$this->server['HOST'].":".$this->server['PORT']."/openmeetings/services/RoomService?wsdl");
			$result = $client_roomService->kickUser($params);
  		}
  		catch(SoapFault $fault){
  			if(defined('MAIL_ADMIN'))
				mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), _('Die Methode kickAll() der Klasse OmGateway wurde aufgrund des folgenden Fehlers abgebrochen: ').$fault->__toString());
  			throw new PortaleException(_("Fehler beim Zugriff auf den Openmeetings-Server. Bitte wenden Sie sich an den Systemadministrator."));
  		}
  	}
}